﻿Public Class Form1
    Private FileName As String = IO.Path.Combine(Application.StartupPath, "AdoxDatabase.mdb")
    Private htTypes As Hashtable = GetOleDbTypeNamesValues(False)
    Private Builder As New OleDb.OleDbConnectionStringBuilder With
        {
            .Provider = "Microsoft.Jet.OLEDB.4.0",
            .DataSource = FileName
        }
    WithEvents bsDataSource As New BindingSource
    Sub CreateNewAccessDatabase(ByVal FileName As String)
        ' The following will show up as an error until you build the project.
        Dim dbCatalog As New ADOX.Catalog()
        dbCatalog.Create(Builder.ConnectionString)
    End Sub
    Private Sub Form1_Load(
        ByVal sender As System.Object,
        ByVal e As System.EventArgs) Handles MyBase.Load

        If IO.File.Exists(FileName) Then
            IO.File.Delete(FileName)
        End If

        cmdFilter.Text = "Set Filter"
        cmdFilter.Enabled = False

    End Sub
    Private Sub cmdCreateUsingADOX_Click(
        ByVal sender As System.Object,
        ByVal e As System.EventArgs) Handles cmdCreateUsingADOX.Click

        If Not IO.File.Exists(FileName) Then
            CreateNewAccessDatabase(FileName)
        End If

        Using cn As New OleDb.OleDbConnection(Builder.ConnectionString)
            Dim cmd As New OleDb.OleDbCommand(
            <text>
                CREATE TABLE persons ([Identifier] int identity ,
                                      UserName NVarchar(50), 
                                      [Password] NVarchar(50),
                                      CONSTRAINT [pk_AutoId] PRIMARY KEY ([Identifier])) 
            </text>.Value, cn)

            cn.Open()
            cmd.ExecuteNonQuery()

            Dim Inserts As New List(Of String) From _
                {
                    "INSERT INTO persons (UserName,[Password]) VALUES ('Gallagher','pass1word')",
                    "INSERT INTO persons (UserName,[Password]) VALUES ('Jones','Fluffy2')",
                    "INSERT INTO persons (UserName,[Password]) VALUES ('Smith','abv123w')",
                    "INSERT INTO persons (UserName,[Password]) VALUES ('Student1','College')",
                    "INSERT INTO persons (UserName,[Password]) VALUES ('Gallagher','pass1word')",
                    "INSERT INTO persons (UserName,[Password]) VALUES ('Simpson','Homer1')"
                }


            For Each Sql As String In Inserts
                cmd.CommandText = Sql
                cmd.ExecuteNonQuery()
            Next

            cmd.CommandText = "SELECT * FROM Persons ORDER BY UserName"

            Dim dt As New DataTable
            dt.Load(cmd.ExecuteReader())
            bsDataSource.DataSource = dt
            DataGridView1.DataSource = bsDataSource
        End Using

        GetOledbSchema(Builder.ConnectionString, "Persons", Me.htTypes, DataGridView2)

        cmdCreateUsingADOX.Enabled = False
        cmdFilter.Enabled = True
    End Sub
    Function GetOleDbTypeNamesValues(ByVal DisplayInformation As Boolean) As Hashtable
        Static Names As String()
        Dim Values As Integer()
        Dim FieldTypes As New Hashtable

        If IsNothing(Names) Then

            Names = CType([Enum].GetNames(GetType(OleDb.OleDbType)), String())
            Values = CType([Enum].GetValues(GetType(OleDb.OleDbType)), Integer())

            For Row As Integer = 0 To Names.Count - 1
                FieldTypes.Add(Values(Row), Names(Row))
            Next

            If DisplayInformation Then

                Dim query = From x In Names.Select(
                            Function(Name, index) String.Format("{0} {1}", index + 1, Name))

                For Each item In query
                    Console.WriteLine(item)
                Next

            End If

        End If

        Return FieldTypes

    End Function
    Public Function GetOledbSchema(ByVal ConnectionString As String, ByVal TableName As String, ByVal FieldTypes As Hashtable, ByVal GridView As DataGridView) As Boolean

        Try
            Using cn As New OleDb.OleDbConnection(ConnectionString)
                Dim Table As New DataTable With {.TableName = String.Format("tbl{0}", TableName)}
                cn.Open()

                Table = cn.GetOleDbSchemaTable(OleDb.OleDbSchemaGuid.Columns, New Object() {Nothing, Nothing, TableName, Nothing})

                Dim dt As DataTable = Table.DefaultView.ToTable(
                    String.Format("{0}_schema", TableName.SpacesToUnderScore), True, New String() _
                        {
                            "Column_Name",
                            "Data_Type",
                            "is_nullable",
                            "Character_Maximum_Length",
                            "Description"
                        }
                    )

                ' Fields which may be null are casted as Object even though we could do various forms of checks and conversions 
                ' this is better as our DataGridView treats items as objects. A good example is the 'Description' field which
                ' more times than not will not be populated.
                Dim query = _
                   ( _
                      From fd In dt.AsEnumerable
                      Select New With
                      {
                            .Column = fd.Field(Of String)("Column_Name"),
                            .DataType = FieldTypes(fd.Field(Of Integer)("Data_Type")),
                            .Size = fd.Field(Of Object)("Character_Maximum_Length"),
                            .Description = fd.Field(Of Object)("Description"),
                            .Nullable = fd.Field(Of Boolean)("is_nullable").ToString
                      }).ToList

                cn.Close()

                GridView.DataSource = query

                Return True

            End Using

        Catch ex As Exception
            Return False
        End Try

    End Function

    Private Sub cmdFilter_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdFilter.Click

        If bsDataSource.DataSource IsNot Nothing Then
            If bsDataSource.Filter = "" Then
                bsDataSource.Filter = "UserName like 'G%'"
                cmdFilter.Text = "Remove Filter"
            Else
                bsDataSource.Filter = ""
                cmdFilter.Text = "Set Filter"
            End If
        End If
    End Sub
    Private Sub cmdCloseForm_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdCloseForm.Click
        Close()
    End Sub
End Class
